/* ###################################### */
/* #                                    # */
/* #  Un compilateur pour le langage    # */
/* #        ALGO                        # */
/* #                                    # */
/* # Unite de conbstruction de l'arbre  # */
/* #                 syntaxique         # */
/* #                                    # */
/* # BRUNO Emmanuel 1998                # */
/* # bruno@univ-tln.fr                  # */
/* ###################################### */


/* l'arbre syntaxique (n-aire) est simule par un arbre binaire */

#include <stdio.h>
#include <string.h>
#include "algo_arbre.h"

noeud creer_noeud(char *nom)
{
  noeud nouveau;
  nouveau = (struct s_noeud *) (malloc(sizeof(struct s_noeud)));
  nouveau->nom=(char *)(malloc(strlen(nom)+1));
  strcpy(nouveau->nom,nom);
  nouveau->frere=NULL;
  nouveau->fils=NULL;
  nouveau->instructions=NULL;
  return nouveau;
};

noeud creer_racine(char *nom)
{
  return creer_noeud(nom);
};

noeud ajoute_fils(noeud racine,noeud nouveau)
{
  noeud courant;
  if (racine==NULL) return nouveau;
  else
    if (racine->fils==NULL) racine->fils=nouveau;
    else 
	{
	  courant=racine->fils;
	  while (courant->frere != NULL) courant=courant->frere;
	courant->frere=nouveau;
	}
  return racine;
}

noeud ajoute_frere(noeud racine,noeud nouveau)
{
  noeud courant;
  courant=racine;
  if (racine==NULL) return nouveau;
  while (courant->frere != NULL) courant = courant->frere;
  courant->frere=nouveau;
  return racine;  
}

void espaces(int nb)
{
  int i;
  for(i=0;i<nb;i++) printf(" ");
}

void affiche_arbre_syntaxique1(noeud racine,int decalage)
{
  noeud courant=racine;
  if (racine != NULL) 
    {
      espaces(decalage);printf("< %s >\n",racine->nom);
      affiche_arbre_syntaxique1(racine->fils,decalage+1);
      affiche_arbre_syntaxique1(racine->frere,decalage);
    }
}

void affiche_arbre_syntaxique(noeud racine)
{
  noeud courant=racine;
  printf("--- Affichage de l'arbre syntaxique ---\n");
  affiche_arbre_syntaxique1(racine,0);
}

void ajoute_instruction(noeud racine, char * nom,char * param1,char * param2,char * param3)
{
  code instruction,nouvelle;
  nouvelle->nom=(char *)(malloc(strlen(nom)+1));strcpy(nouvelle->nom,nom);
  nouvelle->param1=(char *)(malloc(strlen(param1)+1));strcpy(nouvelle->param1,param1);
  nouvelle->param2=(char *)(malloc(strlen(param2)+1));strcpy(nouvelle->param2,param2);
  nouvelle->param3=(char *)(malloc(strlen(param3)+1));strcpy(nouvelle->param3,param3);
  nouvelle->suivant=NULL;
  if (racine->instructions == NULL) racine->instructions=nouvelle;
  else
    {
      instruction=racine->instructions;
      while (instruction != NULL) instruction=instruction->suivant;
      instruction->suivant=nouvelle;
    }
}

void affiche_code1(noeud racine,int decalage)
{
  code instruction;
  if (racine!=NULL)
    {
      instruction=racine->instructions;
      while (instruction != NULL) 
	{
	  espaces(decalage);
	  printf("%s (%s , %s , %s)\n",instruction->nom,instruction->param1,instruction->param2,instruction->param3);
	  instruction=instruction->suivant;	  
	}
      affiche_code1(racine->fils,decalage+1);
      affiche_code1(racine->frere,decalage);
    }
}



void affiche_code(noeud racine)
{
  code instruction;
  printf("--- affichage de code genere ---\n");
  affiche_code1(racine,0);
}

void detruire_arbre_syntaxique(noeud racine)
{
  code instruction,pro;
  if (racine!=NULL)
    {
      detruire_arbre_syntaxique(racine->fils);
      detruire_arbre_syntaxique(racine->frere);
      instruction=racine->instructions;
      while (instruction != NULL) 
	{
	  pro=instruction;
	  instruction=instruction->suivant;
	  free(pro);
	}
      free(racine);
    }
}

/*
int main()
{
  noeud racine=NULL,pro=NULL,pro1=NULL;
  pro=creer_racine("Pere");
  racine=ajoute_fils(racine,pro);
  
  pro=creer_racine("Fils1");
  racine=ajoute_fils(racine,pro);
  pro=creer_racine("Fils2");
  racine=ajoute_fils(racine,pro);

  pro=creer_racine("Frere1");
  racine=ajoute_frere(racine,pro);
  pro=creer_racine("Frere2");
  racine=ajoute_frere(racine,pro);

  pro1=creer_racine("Fr2, Fils1");
  pro=ajoute_fils(pro,pro1);
  pro1=creer_racine("Fr2, Fils2");
  pro=ajoute_fils(pro,pro1);

  pro=creer_racine("Frere3");
  racine=ajoute_frere(racine,pro);
  


  affiche_arbre_syntaxique(racine);

  detruire_arbre_syntaxique(racine);

  return 1;
}*/
